@@ -4,8 +4,6 @@
 
 #pragma once
 
-namespace irr
-{
 namespace scene
 {
 
@@ -34,8 +32,7 @@ enum E_DEBUG_SCENE_TYPE
 	EDS_BBOX_ALL = EDS_BBOX | EDS_BBOX_BUFFERS,
 
 	//! Show all debug infos
-	EDS_FULL = 0xffffffff
+	EDS_FULL = 0xffff
 };
 
 } // end namespace scene
-} // end namespace irr
