@@ -8,15 +8,9 @@
 #include "EPrimitiveTypes.h"
 #include "path.h"
 
-namespace io
-{
-class IReadFile;
-} // end namespace io
-
 namespace video
 {
 
-class IVideoDriver;
 class IShaderConstantSetCallBack;
 
 //! Interface making it possible to create and use programs running on the GPU.
