@@ -19,8 +19,6 @@
 // - For uses of getMatrix() you have to use quaternion::getMatrix_transposed instead.
 // #define IRR_TEST_BROKEN_QUATERNION_USE
 
-namespace irr
-{
 namespace core
 {
 
@@ -180,7 +178,7 @@ public:
 	linear interpolation.
 	*/
 	quaternion &slerp(quaternion q1, quaternion q2,
-			f32 time, f32 threshold = .05f);
+			f32 time, f32 threshold = .001f);
 
 	//! Set this quaternion to represent a rotation from angle and axis.
 	/** Axis must be unit length.
@@ -358,8 +356,6 @@ inline void quaternion::getMatrixFast(matrix4 &dest) const
 	dest[13] = 0.f;
 	dest[14] = 0.f;
 	dest[15] = 1.f;
-
-	dest.setDefinitelyIdentityMatrix(false);
 }
 
 /*!
@@ -397,8 +393,6 @@ inline void quaternion::getMatrix(matrix4 &dest,
 	dest[13] = center.Y;
 	dest[14] = center.Z;
 	dest[15] = 1.f;
-
-	dest.setDefinitelyIdentityMatrix(false);
 }
 
 /*!
@@ -471,8 +465,6 @@ inline void quaternion::getMatrix_transposed(matrix4 &dest) const
 	dest[7] = 0.f;
 	dest[11] = 0.f;
 	dest[15] = 1.f;
-
-	dest.setDefinitelyIdentityMatrix(false);
 }
 
 // Inverts this quaternion
@@ -710,4 +702,3 @@ inline core::quaternion &quaternion::rotationFromTo(const vector3df &from, const
 }
 
 } // end namespace core
-} // end namespace irr
