@@ -5,7 +5,7 @@
 
 #include "SMaterialLayer.h"
 #include "coreutil.h"
-#include "CSkinnedMesh.h"
+#include "SkinnedMesh.h"
 #include "IAnimatedMesh.h"
 #include "IReadFile.h"
 #include "irrTypes.h"
@@ -20,6 +20,7 @@
 #include <array>
 #include <cstddef>
 #include <cstring>
+#include <cassert>
 #include <limits>
 #include <memory>
 #include <optional>
@@ -29,8 +30,6 @@
 #include <variant>
 #include <vector>
 
-namespace irr {
-
 /* Notes on the coordinate system.
  *
  * glTF uses a right-handed coordinate system where +Z is the
@@ -305,7 +304,7 @@ SelfType::createNormalizedValuesAccessor(
 	}
 }
 
-template <std::size_t N>
+template <std::size_t N, bool validate>
 std::array<f32, N> SelfType::getNormalizedValues(
 		const NormalizedValuesAccessor<N> &accessor,
 		const std::size_t i)
@@ -313,17 +312,19 @@ std::array<f32, N> SelfType::getNormalizedValues(
 	std::array<f32, N> values;
 	if (std::holds_alternative<Accessor<std::array<u8, N>>>(accessor)) {
 		const auto u8s = std::get<Accessor<std::array<u8, N>>>(accessor).get(i);
-		for (u8 i = 0; i < N; ++i)
-			values[i] = static_cast<f32>(u8s[i]) / std::numeric_limits<u8>::max();
+		for (std::size_t j = 0; j < N; ++j)
+			values[j] = static_cast<f32>(u8s[j]) / std::numeric_limits<u8>::max();
 	} else if (std::holds_alternative<Accessor<std::array<u16, N>>>(accessor)) {
 		const auto u16s = std::get<Accessor<std::array<u16, N>>>(accessor).get(i);
-		for (u8 i = 0; i < N; ++i)
-			values[i] = static_cast<f32>(u16s[i]) / std::numeric_limits<u16>::max();
+		for (std::size_t j = 0; j < N; ++j)
+			values[j] = static_cast<f32>(u16s[j]) / std::numeric_limits<u16>::max();
 	} else {
 		values = std::get<Accessor<std::array<f32, N>>>(accessor).get(i);
-		for (u8 i = 0; i < N; ++i) {
-			if (values[i] < 0 || values[i] > 1)
-				throw std::runtime_error("invalid normalized value");
+		if constexpr (validate) {
+			for (std::size_t j = 0; j < N; ++j) {
+				if (values[j] < 0 || values[j] > 1)
+					throw std::runtime_error("invalid normalized value");
+			}
 		}
 	}
 	return values;
@@ -344,14 +345,15 @@ IAnimatedMesh* SelfType::createMesh(io::IReadFile* file)
 	const char *filename = file->getFileName().c_str();
 	try {
 		tiniergltf::GlTF model = parseGLTF(file);
-		irr_ptr<CSkinnedMesh> mesh(new CSkinnedMesh());
+		irr_ptr<SkinnedMeshBuilder> mesh(new SkinnedMeshBuilder(
+				SkinnedMesh::SourceFormat::GLTF));
 		MeshExtractor extractor(std::move(model), mesh.get());
 		try {
 			extractor.load();
 			for (const auto &warning : extractor.getWarnings()) {
 				os::Printer::log(filename, warning.c_str(), ELL_WARNING);
 			}
-			return mesh.release();
+			return mesh.release()->finalize();
 		} catch (const std::runtime_error &e) {
 			os::Printer::log("error converting gltf to irrlicht mesh", e.what(), ELL_ERROR);
 		}
@@ -361,18 +363,6 @@ IAnimatedMesh* SelfType::createMesh(io::IReadFile* file)
 	return nullptr;
 }
 
-static void transformVertices(std::vector<video::S3DVertex> &vertices, const core::matrix4 &transform)
-{
-	for (auto &vertex : vertices) {
-		// Apply scaling, rotation and rotation (in that order) to the position.
-		transform.transformVect(vertex.Pos);
-		// For the normal, we do not want to apply the translation.
-		vertex.Normal = transform.rotateAndScaleVect(vertex.Normal);
-		// Renormalize (length might have been affected by scaling).
-		vertex.Normal.normalize();
-	}
-}
-
 static void checkIndices(const std::vector<u16> &indices, const std::size_t nVerts)
 {
 	for (u16 index : indices) {
@@ -410,7 +400,7 @@ static video::E_TEXTURE_CLAMP convertTextureWrap(const Wrap wrap) {
 void SelfType::MeshExtractor::addPrimitive(
 		const tiniergltf::MeshPrimitive &primitive,
 		const std::optional<std::size_t> skinIdx,
-		CSkinnedMesh::SJoint *parent)
+		SkinnedMesh::SJoint *parent)
 {
 	auto vertices = getVertices(primitive);
 	if (!vertices.has_value())
@@ -422,9 +412,6 @@ void SelfType::MeshExtractor::addPrimitive(
 	if (n_vertices >= std::numeric_limits<u16>::max())
 		throw std::runtime_error("too many vertices");
 
-	// Apply the global transform along the parent chain.
-	transformVertices(*vertices, parent->GlobalMatrix);
-
 	auto maybeIndices = getIndices(primitive);
 	std::vector<u16> indices;
 	if (maybeIndices.has_value()) {
@@ -435,10 +422,9 @@ void SelfType::MeshExtractor::addPrimitive(
 		indices = generateIndices(vertices->size());
 	}
 
-	m_irr_model->addMeshBuffer(
-			new SSkinMeshBuffer(std::move(*vertices), std::move(indices)));
+	auto *meshbuf = new SSkinMeshBuffer(std::move(*vertices), std::move(indices));
+	m_irr_model->addMeshBuffer(meshbuf);
 	const auto meshbufNr = m_irr_model->getMeshBufferCount() - 1;
-	auto *meshbuf = m_irr_model->getMeshBuffer(meshbufNr);
 
 	if (primitive.material.has_value()) {
 		const auto &material = m_gltf_model.materials->at(*primitive.material);
@@ -457,17 +443,17 @@ void SelfType::MeshExtractor::addPrimitive(
 		}
 	}
 
-	if (!skinIdx.has_value()) {
-		// No skin => all vertices belong entirely to their parent
-		for (std::size_t v = 0; v < n_vertices; ++v) {
-			auto *weight = m_irr_model->addWeight(parent);
-			weight->buffer_id = meshbufNr;
-			weight->vertex_id = v;
-			weight->strength = 1.0f;
-		}
+	if (!skinIdx) {
+		// Apply the global transform along the parent chain.
+		meshbuf->Transformation = parent->GlobalMatrix;
+		// Set up rigid animation
+		parent->AttachedMeshes.push_back(meshbufNr);
 		return;
 	}
 
+	// Otherwise: "Only the joint transforms are applied to the skinned mesh;
+	// the transform of the skinned mesh node MUST be ignored."
+
 	const auto &skin = m_gltf_model.skins->at(*skinIdx);
 
 	const auto &attrs = primitive.attributes;
@@ -493,6 +479,7 @@ void SelfType::MeshExtractor::addPrimitive(
 
 		const auto weightAccessor = createNormalizedValuesAccessor<4>(m_gltf_model, weights->at(set));
 
+		bool negative_weights = false;
 		for (std::size_t v = 0; v < n_vertices; ++v) {
 			std::array<u16, 4> jointIdxs;
 			if (std::holds_alternative<Accessor<std::array<u8, 4>>>(jointAccessor)) {
@@ -501,21 +488,27 @@ void SelfType::MeshExtractor::addPrimitive(
 			} else if (std::holds_alternative<Accessor<std::array<u16, 4>>>(jointAccessor)) {
 				jointIdxs = std::get<Accessor<std::array<u16, 4>>>(jointAccessor).get(v);
 			}
-			std::array<f32, 4> strengths = getNormalizedValues(weightAccessor, v);
+
+			// Be lax: We can allow weights that aren't normalized. Irrlicht already normalizes them.
+			// The glTF spec only requires that these be "as close to 1 as reasonably possible".
+			auto strengths = getNormalizedValues<4, false>(weightAccessor, v);
 
 			// 4 joints per set
 			for (std::size_t in_set = 0; in_set < 4; ++in_set) {
 				u16 jointIdx = jointIdxs[in_set];
 				f32 strength = strengths[in_set];
-				if (strength == 0)
-					continue;
+				negative_weights = negative_weights || (strength < 0);
+				if (strength <= 0)
+					continue; // note: also ignores negative weights
 
-				CSkinnedMesh::SWeight *weight = m_irr_model->addWeight(m_loaded_nodes.at(skin.joints.at(jointIdx)));
+				SkinnedMesh::SWeight *weight = m_irr_model->addWeight(m_loaded_nodes.at(skin.joints.at(jointIdx)));
 				weight->buffer_id = meshbufNr;
 				weight->vertex_id = v;
 				weight->strength = strength;
 			}
 		}
+		if (negative_weights)
+			warn("negative weights");
 	}
 }
 
@@ -527,7 +520,7 @@ void SelfType::MeshExtractor::addPrimitive(
 void SelfType::MeshExtractor::deferAddMesh(
 		const std::size_t meshIdx,
 		const std::optional<std::size_t> skinIdx,
-		CSkinnedMesh::SJoint *parent)
+		SkinnedMesh::SJoint *parent)
 {
 	m_mesh_loaders.emplace_back([=] {
 		for (std::size_t pi = 0; pi < getPrimitiveCount(meshIdx); ++pi) {
@@ -537,64 +530,36 @@ void SelfType::MeshExtractor::deferAddMesh(
 	});
 }
 
-// Base transformation between left & right handed coordinate systems.
-// This just inverts the Z axis.
-static const core::matrix4 leftToRight = core::matrix4(
-	1, 0, 0, 0,
-	0, 1, 0, 0,
-	0, 0, -1, 0,
-	0, 0, 0, 1
-);
-static const core::matrix4 rightToLeft = leftToRight;
-
-static core::matrix4 loadTransform(const tiniergltf::Node::Matrix &m, CSkinnedMesh::SJoint *joint)
+static core::matrix4 loadTransform(const tiniergltf::Node::Matrix &m, SkinnedMesh::SJoint *joint)
 {
-	// Note: Under the hood, this casts these doubles to floats.
-	core::matrix4 mat = convertHandedness(core::matrix4(
-			m[0], m[1], m[2], m[3],
-			m[4], m[5], m[6], m[7],
-			m[8], m[9], m[10], m[11],
-			m[12], m[13], m[14], m[15]));
-
-	// Decompose the matrix into translation, scale, and rotation.
-	joint->Animatedposition = mat.getTranslation();
-
-	auto scale = mat.getScale();
-	joint->Animatedscale = scale;
-	core::matrix4 inverseScale;
-	inverseScale.setScale(core::vector3df(
-			scale.X == 0 ? 0 : 1 / scale.X,
-			scale.Y == 0 ? 0 : 1 / scale.Y,
-			scale.Z == 0 ? 0 : 1 / scale.Z));
-
-	core::matrix4 axisNormalizedMat = inverseScale * mat;
-	joint->Animatedrotation = axisNormalizedMat.getRotationDegrees();
-	// Invert the rotation because it is applied using `getMatrix_transposed`,
-	// which again inverts.
-	joint->Animatedrotation.makeInverse();
+	core::matrix4 mat;
+	for (size_t i = 0; i < m.size(); ++i)
+		mat[i] = static_cast<f32>(m[i]);
+	mat = convertHandedness(mat);
 
+	// Note: "When a node is targeted for animation [...],
+	// only TRS properties MAY be present; matrix MUST NOT be present."
+	// Thus we MUST NOT do any decomposition, which in general need not exist.
+	joint->transform = mat;
 	return mat;
 }
 
-static core::matrix4 loadTransform(const tiniergltf::Node::TRS &trs, CSkinnedMesh::SJoint *joint)
+static core::matrix4 loadTransform(const tiniergltf::Node::TRS &trs, SkinnedMesh::SJoint *joint)
 {
-	const auto &trans = trs.translation;
-	const auto &rot = trs.rotation;
-	const auto &scale = trs.scale;
-	core::matrix4 transMat;
-	joint->Animatedposition = convertHandedness(core::vector3df(trans[0], trans[1], trans[2]));
-	transMat.setTranslation(joint->Animatedposition);
-	core::matrix4 rotMat;
-	joint->Animatedrotation = convertHandedness(core::quaternion(rot[0], rot[1], rot[2], rot[3]));
-	core::quaternion(joint->Animatedrotation).getMatrix_transposed(rotMat);
-	joint->Animatedscale = core::vector3df(scale[0], scale[1], scale[2]);
-	core::matrix4 scaleMat;
-	scaleMat.setScale(joint->Animatedscale);
-	return transMat * rotMat * scaleMat;
+	const auto &t = trs.translation;
+	const auto &r = trs.rotation;
+	const auto &s = trs.scale;
+	core::Transform transform{
+		convertHandedness(core::vector3df(t[0], t[1], t[2])),
+		convertHandedness(core::quaternion(r[0], r[1], r[2], r[3])),
+		core::vector3df(s[0], s[1], s[2]),
+	};
+	joint->transform = transform;
+	return transform.buildMatrix();
 }
 
 static core::matrix4 loadTransform(std::optional<std::variant<tiniergltf::Node::Matrix, tiniergltf::Node::TRS>> transform,
-		CSkinnedMesh::SJoint *joint) {
+		SkinnedMesh::SJoint *joint) {
 	if (!transform.has_value()) {
 		return core::matrix4();
 	}
@@ -603,13 +568,12 @@ static core::matrix4 loadTransform(std::optional<std::variant<tiniergltf::Node::
 
 void SelfType::MeshExtractor::loadNode(
 		const std::size_t nodeIdx,
-		CSkinnedMesh::SJoint *parent)
+		SkinnedMesh::SJoint *parent)
 {
 	const auto &node = m_gltf_model.nodes->at(nodeIdx);
 	auto *joint = m_irr_model->addJoint(parent);
 	const core::matrix4 transform = loadTransform(node.transform, joint);
-	joint->LocalMatrix = transform;
-	joint->GlobalMatrix = parent ? parent->GlobalMatrix * joint->LocalMatrix : joint->LocalMatrix;
+	joint->GlobalMatrix = parent ? parent->GlobalMatrix * transform : transform;
 	if (node.name.has_value()) {
 		joint->Name = node.name->c_str();
 	}
@@ -626,7 +590,7 @@ void SelfType::MeshExtractor::loadNode(
 
 void SelfType::MeshExtractor::loadNodes()
 {
-	m_loaded_nodes = std::vector<CSkinnedMesh::SJoint *>(m_gltf_model.nodes->size());
+	m_loaded_nodes = std::vector<SkinnedMesh::SJoint *>(m_gltf_model.nodes->size());
 
 	std::vector<bool> isChild(m_gltf_model.nodes->size());
 	for (const auto &node : *m_gltf_model.nodes) {
@@ -666,10 +630,18 @@ void SelfType::MeshExtractor::loadAnimation(const std::size_t animIdx)
 {
 	const auto &anim = m_gltf_model.animations->at(animIdx);
 	for (const auto &channel : anim.channels) {
-
 		const auto &sampler = anim.samplers.at(channel.sampler);
-		if (sampler.interpolation != tiniergltf::AnimationSampler::Interpolation::LINEAR)
-			throw std::runtime_error("unsupported interpolation, only linear interpolation is supported");
+
+		bool interpolate = ([&]() {
+			switch (sampler.interpolation) {
+				case tiniergltf::AnimationSampler::Interpolation::STEP:
+					return false;
+				case tiniergltf::AnimationSampler::Interpolation::LINEAR:
+					return true;
+				default:
+					throw std::runtime_error("Only STEP and LINEAR keyframe interpolation are supported");
+			}
+		})();
 
 		const auto inputAccessor = Accessor<f32>::make(m_gltf_model, sampler.input);
 		const auto n_frames = inputAccessor.getCount();
@@ -677,32 +649,43 @@ void SelfType::MeshExtractor::loadAnimation(const std::size_t animIdx)
 		if (!channel.target.node.has_value())
 			throw std::runtime_error("no animated node");
 
-		const auto &joint = m_loaded_nodes.at(*channel.target.node);
+		auto *joint = m_loaded_nodes.at(*channel.target.node);
+		if (std::holds_alternative<core::matrix4>(joint->transform)) {
+			warn("nodes using matrix transforms must not be animated");
+			continue;
+		}
+
 		switch (channel.target.path) {
 		case tiniergltf::AnimationChannelTarget::Path::TRANSLATION: {
 			const auto outputAccessor = Accessor<core::vector3df>::make(m_gltf_model, sampler.output);
+			auto &channel = joint->keys.position;
+			channel.interpolate = interpolate;
 			for (std::size_t i = 0; i < n_frames; ++i) {
-				auto *key = m_irr_model->addPositionKey(joint);
-				key->frame = inputAccessor.get(i);
-				key->position = convertHandedness(outputAccessor.get(i));
+				f32 frame = inputAccessor.get(i);
+				core::vector3df position = outputAccessor.get(i);
+				channel.pushBack(frame, convertHandedness(position));
 			}
 			break;
 		}
 		case tiniergltf::AnimationChannelTarget::Path::ROTATION: {
 			const auto outputAccessor = Accessor<core::quaternion>::make(m_gltf_model, sampler.output);
+			auto &channel = joint->keys.rotation;
+			channel.interpolate = interpolate;
 			for (std::size_t i = 0; i < n_frames; ++i) {
-				auto *key = m_irr_model->addRotationKey(joint);
-				key->frame = inputAccessor.get(i);
-				key->rotation = convertHandedness(outputAccessor.get(i));
+				f32 frame = inputAccessor.get(i);
+				core::quaternion rotation = outputAccessor.get(i);
+				channel.pushBack(frame, convertHandedness(rotation));
 			}
 			break;
 		}
 		case tiniergltf::AnimationChannelTarget::Path::SCALE: {
 			const auto outputAccessor = Accessor<core::vector3df>::make(m_gltf_model, sampler.output);
+			auto &channel = joint->keys.scale;
+			channel.interpolate = interpolate;
 			for (std::size_t i = 0; i < n_frames; ++i) {
-				auto *key = m_irr_model->addScaleKey(joint);
-				key->frame = inputAccessor.get(i);
-				key->scale = outputAccessor.get(i);
+				f32 frame = inputAccessor.get(i);
+				core::vector3df scale = outputAccessor.get(i);
+				channel.pushBack(frame, scale);
 			}
 			break;
 		}
@@ -747,8 +730,6 @@ void SelfType::MeshExtractor::load()
 	} catch (const std::bad_optional_access &e) {
 		throw std::runtime_error(e.what());
 	}
-
-	m_irr_model->finalize();
 }
 
 /**
@@ -791,7 +772,8 @@ std::optional<std::vector<u16>> SelfType::MeshExtractor::getIndices(
 			index = std::get<Accessor<u16>>(accessor).get(elemIdx);
 			if (index == std::numeric_limits<u16>::max())
 				throw std::runtime_error("invalid index");
-		} else if (std::holds_alternative<Accessor<u32>>(accessor)) {
+		} else {
+			assert(std::holds_alternative<Accessor<u32>>(accessor));
 			u32 indexWide = std::get<Accessor<u32>>(accessor).get(elemIdx);
 			// Use >= here for consistency.
 			if (indexWide >= std::numeric_limits<u16>::max())
@@ -936,5 +918,3 @@ tiniergltf::GlTF SelfType::parseGLTF(io::IReadFile* file)
 }
 
 } // namespace scene
-
-} // namespace irr
