@@ -4,16 +4,9 @@
 
 #include "CLogger.h"
 
-namespace irr
-{
-
 CLogger::CLogger(IEventReceiver *r) :
 		LogLevel(ELL_INFORMATION), Receiver(r)
-{
-#ifdef _DEBUG
-	setDebugName("CLogger");
-#endif
-}
+{}
 
 //! Returns the current set log level.
 ELOG_LEVEL CLogger::getLogLevel() const
@@ -62,5 +55,3 @@ void CLogger::setReceiver(IEventReceiver *r)
 {
 	Receiver = r;
 }
-
-} // end namespace irr
