@@ -6,8 +6,6 @@
 
 #include "IMeshManipulator.h"
 
-namespace irr
-{
 namespace scene
 {
 
@@ -31,16 +29,6 @@ public:
 
 	//! Clones a static IMesh into a modifiable SMesh.
 	SMesh *createMeshCopy(scene::IMesh *mesh) const override;
-
-	//! Returns amount of polygons in mesh.
-	s32 getPolyCount(scene::IMesh *mesh) const override;
-
-	//! Returns amount of polygons in mesh.
-	s32 getPolyCount(scene::IAnimatedMesh *mesh) const override;
-
-	//! create a new AnimatedMesh and adds the mesh to it
-	IAnimatedMesh *createAnimatedMesh(scene::IMesh *mesh, scene::E_ANIMATED_MESH_TYPE type) const override;
 };
 
 } // end namespace scene
-} // end namespace irr
