@@ -3,9 +3,10 @@
 // For conditions of distribution and use, see copyright notice in irrlicht.h
 
 #include "CNullDriver.h"
+#include "IVideoDriver.h"
+#include "SMaterial.h"
 #include "os.h"
 #include "CImage.h"
-#include "CAttributes.h"
 #include "IReadFile.h"
 #include "IWriteFile.h"
 #include "IImageLoader.h"
@@ -17,14 +18,11 @@
 #include "IReferenceCounted.h"
 #include "IRenderTarget.h"
 
-namespace irr
-{
+#include <cassert>
+
 namespace video
 {
 
-//! creates a loader which is able to load windows bitmaps
-IImageLoader *createImageLoaderBMP();
-
 //! creates a loader which is able to load jpeg images
 IImageLoader *createImageLoaderJPG();
 
@@ -56,30 +54,11 @@ CNullDriver::CNullDriver(io::IFileSystem *io, const core::dimension2d<u32> &scre
 		ViewPort(0, 0, 0, 0), ScreenSize(screenSize), MinVertexCountForVBO(500),
 		TextureCreationFlags(0), OverrideMaterial2DEnabled(false), AllowZWriteOnTransparent(false)
 {
-#ifdef _DEBUG
-	setDebugName("CNullDriver");
-#endif
-
-	DriverAttributes = new io::CAttributes();
-	DriverAttributes->addInt("MaxTextures", MATERIAL_MAX_TEXTURES);
-	DriverAttributes->addInt("MaxSupportedTextures", MATERIAL_MAX_TEXTURES);
-	DriverAttributes->addInt("MaxAnisotropy", 1);
-	//	DriverAttributes->addInt("MaxAuxBuffers", 0);
-	DriverAttributes->addInt("MaxMultipleRenderTargets", 1);
-	DriverAttributes->addInt("MaxIndices", -1);
-	DriverAttributes->addInt("MaxTextureSize", -1);
-	//	DriverAttributes->addInt("MaxGeometryVerticesOut", 0);
-	//	DriverAttributes->addFloat("MaxTextureLODBias", 0.f);
-	DriverAttributes->addInt("Version", 1);
-	//	DriverAttributes->addInt("ShaderLanguageVersion", 0);
-	//	DriverAttributes->addInt("AntiAlias", 0);
-
 	setFog();
 
 	setTextureCreationFlag(ETCF_ALWAYS_32_BIT, true);
 	setTextureCreationFlag(ETCF_CREATE_MIP_MAPS, true);
-	setTextureCreationFlag(ETCF_AUTO_GENERATE_MIP_MAPS, true);
-	setTextureCreationFlag(ETCF_ALLOW_MEMORY_COPY, true);
+	setTextureCreationFlag(ETCF_ALLOW_MEMORY_COPY, false);
 
 	ViewPort = core::rect<s32>(core::position2d<s32>(0, 0), core::dimension2di(screenSize));
 
@@ -93,7 +72,6 @@ CNullDriver::CNullDriver(io::IFileSystem *io, const core::dimension2d<u32> &scre
 	SurfaceLoader.push_back(video::createImageLoaderTGA());
 	SurfaceLoader.push_back(video::createImageLoaderPNG());
 	SurfaceLoader.push_back(video::createImageLoaderJPG());
-	SurfaceLoader.push_back(video::createImageLoaderBMP());
 
 	SurfaceWriter.push_back(video::createImageWriterJPG());
 	SurfaceWriter.push_back(video::createImageWriterPNG());
@@ -107,8 +85,16 @@ CNullDriver::CNullDriver(io::IFileSystem *io, const core::dimension2d<u32> &scre
 	InitMaterial2D.ZWriteEnable = video::EZW_OFF;
 	InitMaterial2D.ZBuffer = video::ECFN_DISABLED;
 	InitMaterial2D.UseMipMaps = false;
-	InitMaterial2D.forEachTexture([](auto &tex) {
+	InitMaterial2D.forEachTexture([](video::SMaterialLayer &tex) {
+		// Best preset for 2D pixel-perfect graphics
 		tex.MinFilter = video::ETMINF_NEAREST_MIPMAP_NEAREST;
+
+		// Best preset for downscaled 2D graphics using trilinear interpolation
+		//tex.MinFilter = video::ETMINF_LINEAR_MIPMAP_LINEAR;
+		// Lower bias  -> more crisp images, more jitter
+		// Higher bias -> burry images, less jitter
+		//tex.LODBias = -1;
+
 		tex.MagFilter = video::ETMAGF_NEAREST;
 		tex.TextureWrapU = video::ETC_REPEAT;
 		tex.TextureWrapV = video::ETC_REPEAT;
@@ -120,9 +106,6 @@ CNullDriver::CNullDriver(io::IFileSystem *io, const core::dimension2d<u32> &scre
 //! destructor
 CNullDriver::~CNullDriver()
 {
-	if (DriverAttributes)
-		DriverAttributes->drop();
-
 	if (FileSystem)
 		FileSystem->drop();
 
@@ -226,7 +209,7 @@ bool CNullDriver::beginScene(u16 clearFlag, SColor clearColor, f32 clearDepth, u
 bool CNullDriver::endScene()
 {
 	FPSCounter.registerFrame(os::Timer::getRealTime());
-	updateAllHardwareBuffers();
+	expireHardwareBuffers();
 	updateAllOcclusionQueries();
 	return true;
 }
@@ -243,12 +226,6 @@ bool CNullDriver::queryFeature(E_VIDEO_DRIVER_FEATURE feature) const
 	return false;
 }
 
-//! Get attributes of the actual video driver
-const io::IAttributes &CNullDriver::getDriverAttributes() const
-{
-	return *DriverAttributes;
-}
-
 //! sets transformation
 void CNullDriver::setTransform(E_TRANSFORMATION_STATE state, const core::matrix4 &mat)
 {
@@ -303,25 +280,9 @@ u32 CNullDriver::getTextureCount() const
 
 ITexture *CNullDriver::addTexture(const core::dimension2d<u32> &size, const io::path &name, ECOLOR_FORMAT format)
 {
-	if (0 == name.size()) {
-		os::Printer::log("Could not create ITexture, texture needs to have a non-empty name.", ELL_WARNING);
-		return 0;
-	}
-
 	IImage *image = new CImage(format, size);
-	ITexture *t = 0;
-
-	if (checkImage(image)) {
-		t = createDeviceDependentTexture(name, image);
-	}
-
+	ITexture *t = addTexture(name, image);
 	image->drop();
-
-	if (t) {
-		addTexture(t);
-		t->drop();
-	}
-
 	return t;
 }
 
@@ -338,7 +299,8 @@ ITexture *CNullDriver::addTexture(const io::path &name, IImage *image)
 	ITexture *t = 0;
 
 	if (checkImage(image)) {
-		t = createDeviceDependentTexture(name, image);
+		std::vector tmp { image };
+		t = createDeviceDependentTexture(name, ETT_2D, tmp);
 	}
 
 	if (t) {
@@ -349,6 +311,27 @@ ITexture *CNullDriver::addTexture(const io::path &name, IImage *image)
 	return t;
 }
 
+ITexture *CNullDriver::addArrayTexture(const io::path &name, IImage **images, u32 count)
+{
+	if (0 == name.size()) {
+		os::Printer::log("Could not create ITexture, texture needs to have a non-empty name.", ELL_WARNING);
+		return 0;
+	}
+
+	// this is stupid but who cares
+	std::vector<IImage*> tmp(images, images + count);
+
+	ITexture *t = nullptr;
+	if (checkImage(tmp)) {
+		t = createDeviceDependentTexture(name, ETT_2D_ARRAY, tmp);
+	}
+	if (t) {
+		addTexture(t);
+		t->drop();
+	}
+	return t;
+}
+
 ITexture *CNullDriver::addTextureCubemap(const io::path &name, IImage *imagePosX, IImage *imageNegX, IImage *imagePosY,
 		IImage *imageNegY, IImage *imagePosZ, IImage *imageNegZ)
 {
@@ -366,7 +349,7 @@ ITexture *CNullDriver::addTextureCubemap(const io::path &name, IImage *imagePosX
 	imageArray.push_back(imageNegZ);
 
 	if (checkImage(imageArray)) {
-		t = createDeviceDependentTextureCubemap(name, imageArray);
+		t = createDeviceDependentTexture(name, ETT_CUBEMAP, imageArray);
 	}
 
 	if (t) {
@@ -377,7 +360,7 @@ ITexture *CNullDriver::addTextureCubemap(const io::path &name, IImage *imagePosX
 	return t;
 }
 
-ITexture *CNullDriver::addTextureCubemap(const irr::u32 sideLen, const io::path &name, ECOLOR_FORMAT format)
+ITexture *CNullDriver::addTextureCubemap(const u32 sideLen, const io::path &name, ECOLOR_FORMAT format)
 {
 	if (0 == sideLen)
 		return 0;
@@ -393,7 +376,7 @@ ITexture *CNullDriver::addTextureCubemap(const irr::u32 sideLen, const io::path
 
 	ITexture *t = 0;
 	if (checkImage(imageArray)) {
-		t = createDeviceDependentTextureCubemap(name, imageArray);
+		t = createDeviceDependentTexture(name, ETT_CUBEMAP, imageArray);
 
 		if (t) {
 			addTexture(t);
@@ -414,17 +397,13 @@ ITexture *CNullDriver::getTexture(const io::path &filename)
 	const io::path absolutePath = FileSystem->getAbsolutePath(filename);
 
 	ITexture *texture = findTexture(absolutePath);
-	if (texture) {
-		texture->updateSource(ETS_FROM_CACHE);
+	if (texture)
 		return texture;
-	}
 
 	// Then try the raw filename, which might be in an Archive
 	texture = findTexture(filename);
-	if (texture) {
-		texture->updateSource(ETS_FROM_CACHE);
+	if (texture)
 		return texture;
-	}
 
 	// Now try to open the file using the complete path.
 	io::IReadFile *file = FileSystem->createAndOpenFile(absolutePath);
@@ -438,7 +417,6 @@ ITexture *CNullDriver::getTexture(const io::path &filename)
 		// Re-check name for actual archive names
 		texture = findTexture(file->getFileName());
 		if (texture) {
-			texture->updateSource(ETS_FROM_CACHE);
 			file->drop();
 			return texture;
 		}
@@ -447,7 +425,6 @@ ITexture *CNullDriver::getTexture(const io::path &filename)
 		file->drop();
 
 		if (texture) {
-			texture->updateSource(ETS_FROM_FILE);
 			addTexture(texture);
 			texture->drop(); // drop it because we created it, one grab too much
 		} else
@@ -467,15 +444,12 @@ ITexture *CNullDriver::getTexture(io::IReadFile *file)
 	if (file) {
 		texture = findTexture(file->getFileName());
 
-		if (texture) {
-			texture->updateSource(ETS_FROM_CACHE);
+		if (texture)
 			return texture;
-		}
 
 		texture = loadTextureFromFile(file);
 
 		if (texture) {
-			texture->updateSource(ETS_FROM_FILE);
 			addTexture(texture);
 			texture->drop(); // drop it because we created it, one grab too much
 		}
@@ -497,7 +471,8 @@ video::ITexture *CNullDriver::loadTextureFromFile(io::IReadFile *file, const io:
 		return nullptr;
 
 	if (checkImage(image)) {
-		texture = createDeviceDependentTexture(hashName.size() ? hashName : file->getFileName(), image);
+		std::vector tmp { image };
+		texture = createDeviceDependentTexture(hashName.size() ? hashName : file->getFileName(), ETT_2D, tmp);
 		if (texture)
 			os::Printer::log("Loaded texture", file->getFileName(), ELL_DEBUG);
 	}
@@ -537,18 +512,16 @@ video::ITexture *CNullDriver::findTexture(const io::path &filename)
 	return 0;
 }
 
-ITexture *CNullDriver::createDeviceDependentTexture(const io::path &name, IImage *image)
+ITexture *CNullDriver::createDeviceDependentTexture(const io::path &name, E_TEXTURE_TYPE type,
+		const std::vector<IImage*> &images)
 {
-	SDummyTexture *dummy = new SDummyTexture(name, ETT_2D);
-	dummy->setSize(image->getDimension());
+	if (type != ETT_2D && type != ETT_CUBEMAP)
+		return nullptr;
+	SDummyTexture *dummy = new SDummyTexture(name, type);
+	dummy->setSize(images[0]->getDimension());
 	return dummy;
 }
 
-ITexture *CNullDriver::createDeviceDependentTextureCubemap(const io::path &name, const std::vector<IImage*> &image)
-{
-	return new SDummyTexture(name, ETT_CUBEMAP);
-}
-
 bool CNullDriver::setRenderTargetEx(IRenderTarget *target, u16 clearFlag, SColor clearColor, f32 clearDepth, u8 clearStencil)
 {
 	return false;
@@ -627,20 +600,17 @@ void CNullDriver::draw3DBox(const core::aabbox3d<f32> &box, SColor color)
 	core::vector3df edges[8];
 	box.getEdges(edges);
 
-	// TODO: optimize into one big drawIndexPrimitive call.
+	video::S3DVertex v[8];
+	for (u32 i = 0; i < 8; i++) {
+		v[i].Pos = edges[i];
+		v[i].Color = color;
+	}
 
-	draw3DLine(edges[5], edges[1], color);
-	draw3DLine(edges[1], edges[3], color);
-	draw3DLine(edges[3], edges[7], color);
-	draw3DLine(edges[7], edges[5], color);
-	draw3DLine(edges[0], edges[2], color);
-	draw3DLine(edges[2], edges[6], color);
-	draw3DLine(edges[6], edges[4], color);
-	draw3DLine(edges[4], edges[0], color);
-	draw3DLine(edges[1], edges[0], color);
-	draw3DLine(edges[3], edges[2], color);
-	draw3DLine(edges[7], edges[6], color);
-	draw3DLine(edges[5], edges[4], color);
+	const static u16 box_indices[24] = {
+		5, 1, 1, 3, 3, 7, 7, 5, 0, 2, 2, 6, 6, 4, 4, 0, 1, 0, 3, 2, 7, 6, 5, 4
+	};
+
+	drawVertexPrimitiveList(v, 8, box_indices, 12, EVT_STANDARD, scene::EPT_LINES);
 }
 
 //! draws an 2d image
@@ -664,7 +634,7 @@ void CNullDriver::draw2DImageBatch(const video::ITexture *texture,
 		SColor color,
 		bool useAlphaChannelOfTexture)
 {
-	const irr::u32 drawCount = core::min_<u32>(positions.size(), sourceRects.size());
+	const u32 drawCount = core::min_<u32>(positions.size(), sourceRects.size());
 
 	for (u32 i = 0; i < drawCount; ++i) {
 		draw2DImage(texture, positions[i], sourceRects[i],
@@ -747,19 +717,6 @@ SFrameStats CNullDriver::getFrameStats() const
 	return FrameStats;
 }
 
-//! Sets the dynamic ambient light color. The default color is
-//! (0,0,0,0) which means it is dark.
-//! \param color: New color of the ambient light.
-void CNullDriver::setAmbientLight(const SColorf &color)
-{
-	AmbientLight = color;
-}
-
-const SColorf &CNullDriver::getAmbientLight() const
-{
-	return AmbientLight;
-}
-
 //! \return Returns the name of the video driver. Example: In case of the DIRECT3D8
 //! driver, it would return "Direct3D8".
 
@@ -917,6 +874,9 @@ bool CNullDriver::checkImage(const std::vector<IImage*> &image) const
 	auto lastSize = image[0]->getDimension();
 
 	for (size_t i = 0; i < image.size(); ++i) {
+		if (!image[i])
+			return false;
+
 		ECOLOR_FORMAT format = image[i]->getColorFormat();
 		auto size = image[i]->getDimension();
 
@@ -940,9 +900,10 @@ void CNullDriver::setTextureCreationFlag(E_TEXTURE_CREATION_FLAG flag, bool enab
 		setTextureCreationFlag(ETCF_OPTIMIZED_FOR_SPEED, false);
 	}
 
-	// set flag
-	TextureCreationFlags = (TextureCreationFlags & (~flag)) |
-						   ((((u32)!enabled) - 1) & flag);
+	if (enabled)
+		TextureCreationFlags |= flag;
+	else
+		TextureCreationFlags &= ~flag;
 }
 
 //! Returns if a texture creation flag is enabled or disabled.
@@ -1117,7 +1078,7 @@ void CNullDriver::drawBuffers(const scene::IVertexBuffer *vb,
 
 	if (vb->getHWBuffer() || ib->getHWBuffer()) {
 		// subclass is supposed to override this if it supports hw buffers
-		_IRR_DEBUG_BREAK_IF(1);
+		assert(false);
 	}
 
 	drawVertexPrimitiveList(vb->getData(), vb->getCount(), ib->getData(),
@@ -1161,32 +1122,68 @@ CNullDriver::SHWBufferLink *CNullDriver::getBufferLink(const scene::IIndexBuffer
 	return createHardwareBuffer(ib); // no hardware links, and mesh wants one, create it
 }
 
-//! Update all hardware buffers, remove unused ones
-void CNullDriver::updateAllHardwareBuffers()
+void CNullDriver::registerHardwareBuffer(SHWBufferLink *HWBuffer)
 {
-	auto it = HWBufferList.begin();
-	while (it != HWBufferList.end()) {
-		SHWBufferLink *Link = *it;
-		++it;
+	assert(HWBuffer);
+	HWBuffer->ListPosition = HWBufferList.size();
+	HWBufferList.push_back(HWBuffer);
+}
 
-		if (Link->IsVertex) {
-			if (!Link->VertexBuffer || Link->VertexBuffer->getReferenceCount() == 1)
-				deleteHardwareBuffer(Link);
-		} else {
-			if (!Link->IndexBuffer || Link->IndexBuffer->getReferenceCount() == 1)
-				deleteHardwareBuffer(Link);
-		}
+void CNullDriver::expireHardwareBuffers()
+{
+	for (size_t i = 0; i < HWBufferList.size(); ) {
+		auto *Link = HWBufferList[i];
+
+		bool del;
+		if (Link->IsVertex)
+			del = !Link->VertexBuffer || Link->VertexBuffer->getReferenceCount() == 1;
+		else
+			del = !Link->IndexBuffer || Link->IndexBuffer->getReferenceCount() == 1;
+		// deleting can reorder, so don't advance in list
+		if (del)
+			deleteHardwareBuffer(Link);
+		else
+			i++;
 	}
+
+	FrameStats.HWBuffersActive = HWBufferList.size();
 }
 
 void CNullDriver::deleteHardwareBuffer(SHWBufferLink *HWBuffer)
 {
 	if (!HWBuffer)
 		return;
-	HWBufferList.erase(HWBuffer->listPosition);
+	const size_t pos = HWBuffer->ListPosition;
+	assert(HWBufferList.at(pos) == HWBuffer);
+	if (HWBufferList.size() < 2 || pos == HWBufferList.size() - 1) {
+		HWBufferList.erase(HWBufferList.begin() + pos);
+	} else {
+		// swap with last
+		std::swap(HWBufferList[pos], HWBufferList.back());
+		HWBufferList.pop_back();
+		HWBufferList[pos]->ListPosition = pos;
+	}
 	delete HWBuffer;
 }
 
+void CNullDriver::updateHardwareBuffer(const scene::IVertexBuffer *vb)
+{
+	if (!vb)
+		return;
+	auto *link = getBufferLink(vb);
+	if (link)
+		updateHardwareBuffer(link);
+}
+
+void CNullDriver::updateHardwareBuffer(const scene::IIndexBuffer *ib)
+{
+	if (!ib)
+		return;
+	auto *link = getBufferLink(ib);
+	if (link)
+		updateHardwareBuffer(link);
+}
+
 void CNullDriver::removeHardwareBuffer(const scene::IVertexBuffer *vb)
 {
 	if (!vb)
@@ -1247,7 +1244,7 @@ void CNullDriver::addOcclusionQuery(scene::ISceneNode *node, const scene::IMesh
 		else if (node->getType() == scene::ESNT_MESH)
 			mesh = static_cast<scene::IMeshSceneNode *>(node)->getMesh();
 		else
-			mesh = static_cast<scene::IAnimatedMeshSceneNode *>(node)->getMesh()->getMesh(0);
+			mesh = static_cast<scene::IAnimatedMeshSceneNode *>(node)->getMesh();
 		if (!mesh)
 			return;
 	}
@@ -1298,7 +1295,7 @@ void CNullDriver::runOcclusionQuery(scene::ISceneNode *node, bool visible)
 	OcclusionQueries[index].Run = 0;
 	if (!visible) {
 		SMaterial mat;
-		mat.AntiAliasing = 0;
+		mat.AntiAliasing = video::EAAM_OFF;
 		mat.ColorMask = ECP_NONE;
 		mat.ZWriteEnable = EZW_OFF;
 		setMaterial(mat);
@@ -1442,9 +1439,9 @@ void CNullDriver::setMaterialRendererName(u32 idx, const char *name)
 void CNullDriver::swapMaterialRenderers(u32 idx1, u32 idx2, bool swapNames)
 {
 	if (idx1 < MaterialRenderers.size() && idx2 < MaterialRenderers.size()) {
-		irr::core::swap(MaterialRenderers[idx1].Renderer, MaterialRenderers[idx2].Renderer);
+		std::swap(MaterialRenderers[idx1].Renderer, MaterialRenderers[idx2].Renderer);
 		if (swapNames)
-			irr::core::swap(MaterialRenderers[idx1].Name, MaterialRenderers[idx2].Name);
+			std::swap(MaterialRenderers[idx1].Name, MaterialRenderers[idx2].Name);
 	}
 }
 
@@ -1504,34 +1501,24 @@ IGPUProgrammingServices *CNullDriver::getGPUProgrammingServices()
 //! Adds a new material renderer to the VideoDriver, based on a high level shading language.
 s32 CNullDriver::addHighLevelShaderMaterial(
 		const c8 *vertexShaderProgram,
-		const c8 *vertexShaderEntryPointName,
-		E_VERTEX_SHADER_TYPE vsCompileTarget,
 		const c8 *pixelShaderProgram,
-		const c8 *pixelShaderEntryPointName,
-		E_PIXEL_SHADER_TYPE psCompileTarget,
 		const c8 *geometryShaderProgram,
-		const c8 *geometryShaderEntryPointName,
-		E_GEOMETRY_SHADER_TYPE gsCompileTarget,
+		const c8 *shaderName,
 		scene::E_PRIMITIVE_TYPE inType, scene::E_PRIMITIVE_TYPE outType,
 		u32 verticesOut,
 		IShaderConstantSetCallBack *callback,
 		E_MATERIAL_TYPE baseMaterial,
 		s32 userData)
 {
-	os::Printer::log("High level shader materials not available (yet) in this driver, sorry");
+	os::Printer::log("Shader materials not available in this driver", ELL_ERROR);
 	return -1;
 }
 
 s32 CNullDriver::addHighLevelShaderMaterialFromFiles(
 		const io::path &vertexShaderProgramFileName,
-		const c8 *vertexShaderEntryPointName,
-		E_VERTEX_SHADER_TYPE vsCompileTarget,
 		const io::path &pixelShaderProgramFileName,
-		const c8 *pixelShaderEntryPointName,
-		E_PIXEL_SHADER_TYPE psCompileTarget,
 		const io::path &geometryShaderProgramFileName,
-		const c8 *geometryShaderEntryPointName,
-		E_GEOMETRY_SHADER_TYPE gsCompileTarget,
+		const c8 *shaderName,
 		scene::E_PRIMITIVE_TYPE inType, scene::E_PRIMITIVE_TYPE outType,
 		u32 verticesOut,
 		IShaderConstantSetCallBack *callback,
@@ -1567,9 +1554,7 @@ s32 CNullDriver::addHighLevelShaderMaterialFromFiles(
 	}
 
 	s32 result = addHighLevelShaderMaterialFromFiles(
-			vsfile, vertexShaderEntryPointName, vsCompileTarget,
-			psfile, pixelShaderEntryPointName, psCompileTarget,
-			gsfile, geometryShaderEntryPointName, gsCompileTarget,
+			vsfile, psfile, gsfile, shaderName,
 			inType, outType, verticesOut,
 			callback, baseMaterial, userData);
 
@@ -1587,14 +1572,9 @@ s32 CNullDriver::addHighLevelShaderMaterialFromFiles(
 
 s32 CNullDriver::addHighLevelShaderMaterialFromFiles(
 		io::IReadFile *vertexShaderProgram,
-		const c8 *vertexShaderEntryPointName,
-		E_VERTEX_SHADER_TYPE vsCompileTarget,
 		io::IReadFile *pixelShaderProgram,
-		const c8 *pixelShaderEntryPointName,
-		E_PIXEL_SHADER_TYPE psCompileTarget,
 		io::IReadFile *geometryShaderProgram,
-		const c8 *geometryShaderEntryPointName,
-		E_GEOMETRY_SHADER_TYPE gsCompileTarget,
+		const c8 *shaderName,
 		scene::E_PRIMITIVE_TYPE inType, scene::E_PRIMITIVE_TYPE outType,
 		u32 verticesOut,
 		IShaderConstantSetCallBack *callback,
@@ -1640,9 +1620,7 @@ s32 CNullDriver::addHighLevelShaderMaterialFromFiles(
 	}
 
 	s32 result = this->addHighLevelShaderMaterial(
-			vs, vertexShaderEntryPointName, vsCompileTarget,
-			ps, pixelShaderEntryPointName, psCompileTarget,
-			gs, geometryShaderEntryPointName, gsCompileTarget,
+			vs, ps, gs, shaderName,
 			inType, outType, verticesOut,
 			callback, baseMaterial, userData);
 
@@ -1681,7 +1659,13 @@ ITexture *CNullDriver::addRenderTargetTexture(const core::dimension2d<u32> &size
 	return 0;
 }
 
-ITexture *CNullDriver::addRenderTargetTextureCubemap(const irr::u32 sideLen,
+ITexture *CNullDriver::addRenderTargetTextureMs(const core::dimension2d<u32> &size, u8 msaa,
+		const io::path &name, const ECOLOR_FORMAT format)
+{
+	return 0;
+}
+
+ITexture *CNullDriver::addRenderTargetTextureCubemap(const u32 sideLen,
 		const io::path &name, const ECOLOR_FORMAT format)
 {
 	return 0;
@@ -1753,7 +1737,7 @@ core::dimension2du CNullDriver::getMaxTextureSize() const
 	return core::dimension2du(0x10000, 0x10000); // maybe large enough
 }
 
-bool CNullDriver::needsTransparentRenderPass(const irr::video::SMaterial &material) const
+bool CNullDriver::needsTransparentRenderPass(const video::SMaterial &material) const
 {
 	// TODO: I suspect it would be nice if the material had an enum for further control.
 	//		Especially it probably makes sense to allow disabling transparent render pass as soon as material.ZWriteEnable is on.
@@ -1771,21 +1755,4 @@ bool CNullDriver::needsTransparentRenderPass(const irr::video::SMaterial &materi
 	return false;
 }
 
-//! Color conversion convenience function
-/** Convert an image (as array of pixels) from source to destination
-array, thereby converting the color format. The pixel size is
-determined by the color formats.
-\param sP Pointer to source
-\param sF Color format of source
-\param sN Number of pixels to convert, both array must be large enough
-\param dP Pointer to destination
-\param dF Color format of destination
-*/
-void CNullDriver::convertColor(const void *sP, ECOLOR_FORMAT sF, s32 sN,
-		void *dP, ECOLOR_FORMAT dF) const
-{
-	video::CColorConverter::convert_viaFormat(sP, sF, sN, dP, dF);
-}
-
-} // end namespace
 } // end namespace
