@@ -6,29 +6,24 @@
 
 #include "irrTypes.h"
 
-namespace irr
-{
 namespace video
 {
 
 class COpenGLCoreFeature
 {
 public:
-	COpenGLCoreFeature() :
-			BlendOperation(false), ColorAttachment(0), MultipleRenderTarget(0), MaxTextureUnits(1)
-	{
-	}
+	COpenGLCoreFeature() = default;
 
 	virtual ~COpenGLCoreFeature()
 	{
 	}
 
-	bool BlendOperation;
+	bool BlendOperation = false;
+	bool TexStorage = false;
 
-	u8 ColorAttachment;
-	u8 MultipleRenderTarget;
-	u8 MaxTextureUnits;
+	u8 ColorAttachment = 0;
+	u8 MultipleRenderTarget = 0;
+	u8 MaxTextureUnits = 0;
 };
 
 }
-}
