@@ -8,18 +8,12 @@
 
 #include "CIrrDeviceSDL.h"
 
-namespace irr
-{
 namespace video
 {
 
 CSDLManager::CSDLManager(CIrrDeviceSDL *device) :
 		IContextManager(), SDLDevice(device)
-{
-#ifdef _DEBUG
-	setDebugName("CSDLManager");
-#endif
-}
+{}
 
 bool CSDLManager::initialize(const SIrrlichtCreationParameters &params, const SExposedVideoData &data)
 {
@@ -48,7 +42,6 @@ bool CSDLManager::swapBuffers()
 	return true;
 }
 
-}
 }
 
 #endif
