@@ -8,8 +8,6 @@
 #include "ISceneManager.h"
 #include "IVideoDriver.h"
 
-namespace irr
-{
 namespace scene
 {
 
@@ -32,4 +30,3 @@ private:
 };
 
 } // end namespace scene
-} // end namespace irr
