@@ -8,9 +8,6 @@
 #include "IWriteFile.h"
 #include "irrString.h"
 
-namespace irr
-{
-
 namespace io
 {
 
@@ -55,4 +52,3 @@ private:
 };
 
 } // end namespace io
-} // end namespace irr
