@@ -11,8 +11,6 @@
 #include "SMaterial.h"
 #include "irrMath.h"
 
-namespace irr
-{
 namespace scene
 {
 
@@ -68,4 +66,3 @@ struct SB3dMaterial
 };
 
 } // end namespace scene
-} // end namespace irr
