@@ -35,7 +35,6 @@ class ISoundManager;
 class IWritableItemDefManager;
 class IWritableShaderSource;
 class IWritableTextureSource;
-class MapBlockMesh;
 class MapDatabase;
 class MeshUpdateManager;
 class Minimap;
@@ -51,10 +50,13 @@ struct ClientDynamicInfo;
 struct ClientEvent;
 struct MapDrawControl;
 struct MapNode;
-struct MeshMakeData;
-struct MinimapMapblock;
 struct PlayerControl;
 struct PointedThing;
+struct ItemVisualsManager;
+
+namespace scene {
+class IAnimatedMesh;
+}
 
 namespace con {
 class IConnection;
@@ -99,7 +101,6 @@ private:
 };
 
 class ClientScripting;
-class GameUI;
 
 class Client : public con::PeerHandler, public InventoryManager, public IGameDef
 {
@@ -119,7 +120,7 @@ public:
 			ISoundManager *sound,
 			MtEventManager *event,
 			RenderingEngine *rendering_engine,
-			GameUI *game_ui,
+			ItemVisualsManager *item_visuals,
 			ELoginRegister allow_login_or_register
 	);
 
@@ -142,8 +143,7 @@ public:
 
 	bool isShutdown();
 
-	void connect(const Address &address, const std::string &address_name,
-		bool is_local_server);
+	void connect(const Address &address, const std::string &address_name);
 
 	/*
 		Stuff that references the environment is valid only as
@@ -194,6 +194,7 @@ public:
 	void handleCommand_DetachedInventory(NetworkPacket* pkt);
 	void handleCommand_ShowFormSpec(NetworkPacket* pkt);
 	void handleCommand_SpawnParticle(NetworkPacket* pkt);
+	void handleCommand_SpawnParticleBatch(NetworkPacket *pkt);
 	void handleCommand_AddParticleSpawner(NetworkPacket* pkt);
 	void handleCommand_DeleteParticleSpawner(NetworkPacket* pkt);
 	void handleCommand_HudAdd(NetworkPacket* pkt);
@@ -219,6 +220,7 @@ public:
 	void handleCommand_MediaPush(NetworkPacket *pkt);
 	void handleCommand_MinimapModes(NetworkPacket *pkt);
 	void handleCommand_SetLighting(NetworkPacket *pkt);
+	void handleCommand_Camera(NetworkPacket* pkt);
 
 	void ProcessData(NetworkPacket *pkt);
 
@@ -278,7 +280,10 @@ public:
 		return m_env.getPlayerNames();
 	}
 
-	float getAnimationTime();
+	float getAnimationTime() const
+	{
+		return m_animation_time;
+	}
 
 	int getCrackLevel();
 	v3s16 getCrackPos();
@@ -295,15 +300,13 @@ public:
 	bool getChatMessage(std::wstring &message);
 	void typeChatMessage(const std::wstring& message);
 
-	u64 getMapSeed(){ return m_map_seed; }
+	u64 getMapSeed() const { return m_map_seed; }
 
 	void addUpdateMeshTask(v3s16 blockpos, bool ack_to_server=false, bool urgent=false);
 	// Including blocks at appropriate edges
 	void addUpdateMeshTaskWithEdge(v3s16 blockpos, bool ack_to_server=false, bool urgent=false);
 	void addUpdateMeshTaskForNode(v3s16 nodepos, bool ack_to_server=false, bool urgent=false);
 
-	void updateCameraOffset(v3s16 camera_offset);
-
 	bool hasClientEvents() const { return !m_client_event_queue.empty(); }
 	// Get event from queue. If queue is empty, it triggers an assertion failure.
 	ClientEvent * getClientEvent();
@@ -338,8 +341,17 @@ public:
 	u16 getProtoVersion() const
 	{ return m_proto_ver; }
 
+	// Whether the server is in "simple singleplayer mode".
+	// This implies "m_internal_server = true".
 	bool m_simple_singleplayer_mode;
 
+	// Whether the server is hosted by the same Luanti instance and singletons
+	// like g_settings are shared between client and server.
+	//
+	// This is intentionally *not* true if we're just connecting to a localhost
+	// server hosted by a different Luanti instance.
+	bool m_internal_server;
+
 	float mediaReceiveProgress();
 
 	void drawLoadScreen(const std::wstring &text, float dtime, int percent);
@@ -353,13 +365,14 @@ public:
 		return getProtoVersion() != 0; // (set in TOCLIENT_HELLO)
 	}
 
-	Minimap* getMinimap() { return m_minimap; }
+	Minimap* getMinimap() { return m_minimap.get(); }
 	void setCamera(Camera* camera) { m_camera = camera; }
 
 	Camera* getCamera () { return m_camera; }
 	scene::ISceneManager *getSceneManager();
 
 	// IGameDef interface
+	bool isClient() override { return true; }
 	IItemDefManager* getItemDefManager() override;
 	const NodeDefManager* getNodeDefManager() override;
 	ICraftDefManager* getCraftDefManager() override;
@@ -375,6 +388,8 @@ public:
 	const std::string* getModFile(std::string filename);
 	ModStorageDatabase *getModStorageDatabase() override { return m_mod_storage_database; }
 
+	ItemVisualsManager *getItemVisualsManager() { return m_item_visuals_manager; }
+
 	// Migrates away old files-based mod storage if necessary
 	void migrateModStorage();
 
@@ -441,9 +456,7 @@ private:
 	void peerAdded(con::IPeer *peer) override;
 	void deletingPeer(con::IPeer *peer, bool timeout) override;
 
-	void initLocalMapSaving(const Address &address,
-			const std::string &hostname,
-			bool is_local_server);
+	void initLocalMapSaving(const Address &address, const std::string &hostname);
 
 	void ReceiveAll();
 
@@ -474,6 +487,7 @@ private:
 	ISoundManager *m_sound;
 	MtEventManager *m_event;
 	RenderingEngine *m_rendering_engine;
+	ItemVisualsManager *m_item_visuals_manager;
 
 
 	std::unique_ptr<MeshUpdateManager> m_mesh_update_manager;
@@ -483,29 +497,24 @@ private:
 	std::string m_address_name;
 	ELoginRegister m_allow_login_or_register = ELoginRegister::Any;
 	Camera *m_camera = nullptr;
-	Minimap *m_minimap = nullptr;
+	std::unique_ptr<Minimap> m_minimap;
 
 	// Server serialization version
 	u8 m_server_ser_ver;
 
 	// Used version of the protocol with server
-	// Values smaller than 25 only mean they are smaller than 25,
-	// and aren't accurate. We simply just don't know, because
-	// the server didn't send the version back then.
 	// If 0, server init hasn't been received yet.
 	u16 m_proto_ver = 0;
 
 	bool m_update_wielded_item = false;
-	Inventory *m_inventory_from_server = nullptr;
+	std::unique_ptr<Inventory> m_inventory_from_server;
 	float m_inventory_from_server_age = 0.0f;
+	s32 m_mapblock_limit_logged = 0;
 	PacketCounter m_packetcounter;
 	// Block mesh animation parameters
 	float m_animation_time = 0.0f;
 	int m_crack_level = -1;
 	v3s16 m_crack_pos;
-	// 0 <= m_daynight_i < DAYNIGHT_CACHE_COUNT
-	//s32 m_daynight_i;
-	//u32 m_daynight_ratio;
 	std::queue<std::wstring> m_out_chat_queue;
 	u32 m_last_chat_message_sent;
 	float m_chat_message_allowance = 5.0f;
@@ -514,7 +523,7 @@ private:
 	// The authentication methods we can use to enter sudo mode (=change password)
 	u32 m_sudo_auth_methods;
 
-	// The seed returned by the server in TOCLIENT_INIT is stored here
+	// The seed returned by the server in TOCLIENT_AUTH_ACCEPT is stored here
 	u64 m_map_seed = 0;
 
 	// Auth data
@@ -537,15 +546,10 @@ private:
 
 	std::vector<std::string> m_remote_media_servers;
 	// Media downloader, only exists during init
-	ClientMediaDownloader *m_media_downloader;
+	std::unique_ptr<ClientMediaDownloader> m_media_downloader;
 	// Pending downloads of dynamic media (key: token)
 	std::vector<std::pair<u32, std::shared_ptr<SingleMediaDownloader>>> m_pending_media_downloads;
 
-	// time_of_day speed approximation for old protocol
-	bool m_time_of_day_set = false;
-	float m_last_time_of_day_f = -1.0f;
-	float m_time_of_day_update_timer = 0.0f;
-
 	// An interval for generally sending object positions and stuff
 	float m_recommended_send_interval = 0.1f;
 
@@ -572,10 +576,8 @@ private:
 	// own state
 	LocalClientState m_state;
 
-	GameUI *m_game_ui;
-
 	// Used for saving server map to disk client-side
-	MapDatabase *m_localdb = nullptr;
+	std::unique_ptr<MapDatabase> m_localdb;
 	IntervalLimiter m_localdb_save_interval;
 	u16 m_cache_save_interval;
 
