@@ -14,10 +14,7 @@
 class ClientEnvironment;
 class ITextureSource;
 class Client;
-class IGameDef;
-class LocalPlayer;
 struct ItemStack;
-class WieldMeshSceneNode;
 
 namespace scene
 {
