@@ -6,6 +6,7 @@
 
 #include "irrlichttypes.h"
 #include "config.h"
+#include <csignal>
 #include <string>
 
 #if !IS_CLIENT_BUILD
@@ -15,7 +16,6 @@
 class InputHandler;
 class ChatBackend;
 class RenderingEngine;
-struct SubgameSpec;
 struct GameStartData;
 
 struct Jitter {
@@ -36,7 +36,7 @@ struct CameraOrientation {
 #define GAME_FALLBACK_TIMEOUT 1.8f
 #define GAME_CONNECTION_TIMEOUT 10.0f
 
-void the_game(bool *kill,
+void the_game(volatile std::sig_atomic_t *kill,
 		InputHandler *input,
 		RenderingEngine *rendering_engine,
 		const GameStartData &start_data,
