@@ -9,17 +9,15 @@
 #include "settings.h"
 
 RenderingCore::RenderingCore(IrrlichtDevice *_device, Client *_client, Hud *_hud,
-		ShadowRenderer *_shadow_renderer, RenderPipeline *_pipeline, v2f _virtual_size_scale)
-	: device(_device), client(_client), hud(_hud), shadow_renderer(_shadow_renderer),
-	pipeline(_pipeline), virtual_size_scale(_virtual_size_scale)
+		std::unique_ptr<ShadowRenderer> _shadow_renderer,
+		std::unique_ptr<RenderPipeline> _pipeline,
+		v2f _virtual_size_scale)
+	: device(_device), client(_client), hud(_hud), shadow_renderer(std::move(_shadow_renderer)),
+	pipeline(std::move(_pipeline)), virtual_size_scale(_virtual_size_scale)
 {
 }
 
-RenderingCore::~RenderingCore()
-{
-	delete pipeline;
-	delete shadow_renderer;
-}
+RenderingCore::~RenderingCore() = default;
 
 void RenderingCore::draw(video::SColor _skycolor, bool _show_hud,
 		bool _draw_wield_tool, bool _draw_crosshair)
@@ -27,7 +25,7 @@ void RenderingCore::draw(video::SColor _skycolor, bool _show_hud,
 	v2u32 screensize = device->getVideoDriver()->getScreenSize();
 	virtual_size = v2u32(screensize.X * virtual_size_scale.X, screensize.Y * virtual_size_scale.Y);
 
-	PipelineContext context(device, client, hud, shadow_renderer, _skycolor, screensize);
+	PipelineContext context(device, client, hud, shadow_renderer.get(), _skycolor, screensize);
 	context.draw_crosshair = _draw_crosshair;
 	context.draw_wield_tool = _draw_wield_tool;
 	context.show_hud = _show_hud;
