@@ -97,7 +97,7 @@ void Database_PostgreSQL::ping()
 
 bool Database_PostgreSQL::initialized() const
 {
-	return (PQstatus(m_conn) == CONNECTION_OK);
+	return m_conn && PQstatus(m_conn) == CONNECTION_OK;
 }
 
 PGresult *Database_PostgreSQL::checkResults(PGresult *result, bool clear)
@@ -164,9 +164,9 @@ void MapDatabasePostgreSQL::createDatabase()
 {
 	createTableIfNotExists("blocks",
 		"CREATE TABLE blocks ("
-			"posX INT NOT NULL,"
-			"posY INT NOT NULL,"
-			"posZ INT NOT NULL,"
+			"posX smallint NOT NULL,"
+			"posY smallint NOT NULL,"
+			"posZ smallint NOT NULL,"
 			"data BYTEA,"
 			"PRIMARY KEY (posX,posY,posZ)"
 			");"
