@@ -114,7 +114,7 @@ void DungeonGen::generate(MMVManip *vm, u32 bseed, v3s16 nmin, v3s16 nmax)
 		u32 i = vm->m_area.index(nmin.X, y, z);
 		for (s16 x = nmin.X; x <= nmax.X; x++) {
 			if (vm->m_data[i].getContent() == dp.c_wall) {
-				if (NoisePerlin3D(&dp.np_alt_wall, x, y, z, blockseed) > 0.0f)
+				if (NoiseFractal3D(&dp.np_alt_wall, x, y, z, blockseed) > 0.0f)
 					vm->m_data[i].setContent(dp.c_alt_wall);
 			}
 			i++;
@@ -127,7 +127,7 @@ void DungeonGen::generate(MMVManip *vm, u32 bseed, v3s16 nmin, v3s16 nmax)
 
 void DungeonGen::makeDungeon(v3s16 start_padding)
 {
-	const v3s16 &areasize = vm->m_area.getExtent();
+	const v3s32 &areasize = vm->m_area.getExtent();
 	v3s16 roomsize;
 	v3s16 roomplace;
 
