@@ -10,9 +10,8 @@
 ///////// Mapgen V5 flags
 #define MGV5_CAVERNS 0x01
 
-class BiomeManager;
 
-extern FlagDesc flagdesc_mapgen_v5[];
+extern const FlagDesc flagdesc_mapgen_v5[];
 
 struct MapgenV5Params : public MapgenParams
 {
