@@ -9,15 +9,8 @@ extern "C" {
 }
 
 #include <iostream>
-
 #include "exceptions.h"
 
-struct EnumString
-{
-	int num;
-	const char *str;
-};
-
 class StackUnroller
 {
 private:
@@ -41,7 +34,3 @@ class LuaError : public ModError
 public:
 	LuaError(const std::string &s) : ModError(s) {}
 };
-
-
-extern EnumString es_ItemType[];
-extern EnumString es_TouchInteractionMode[];
