@@ -121,7 +121,7 @@ void RollbackManager::registerNewNode(const int id, const std::string &name)
 
 int RollbackManager::getActorId(const std::string &name)
 {
-	for (std::vector<Entity>::const_iterator iter = knownActors.begin();
+	for (auto iter = knownActors.begin();
 			iter != knownActors.end(); ++iter) {
 		if (iter->name == name) {
 			return iter->id;
@@ -141,7 +141,7 @@ int RollbackManager::getActorId(const std::string &name)
 
 int RollbackManager::getNodeId(const std::string &name)
 {
-	for (std::vector<Entity>::const_iterator iter = knownNodes.begin();
+	for (auto iter = knownNodes.begin();
 			iter != knownNodes.end(); ++iter) {
 		if (iter->name == name) {
 			return iter->id;
@@ -161,7 +161,7 @@ int RollbackManager::getNodeId(const std::string &name)
 
 const char * RollbackManager::getActorName(const int id)
 {
-	for (std::vector<Entity>::const_iterator iter = knownActors.begin();
+	for (auto iter = knownActors.begin();
 			iter != knownActors.end(); ++iter) {
 		if (iter->id == id) {
 			return iter->name.c_str();
@@ -174,7 +174,7 @@ const char * RollbackManager::getActorName(const int id)
 
 const char * RollbackManager::getNodeName(const int id)
 {
-	for (std::vector<Entity>::const_iterator iter = knownNodes.begin();
+	for (auto iter = knownNodes.begin();
 			iter != knownNodes.end(); ++iter) {
 		if (iter->id == id) {
 			return iter->name.c_str();
@@ -224,7 +224,12 @@ bool RollbackManager::createTables()
 		"	FOREIGN KEY (`oldNode`)   REFERENCES `node`(`id`),\n"
 		"	FOREIGN KEY (`newNode`)   REFERENCES `node`(`id`)\n"
 		");\n"
-		"CREATE INDEX IF NOT EXISTS `actionIndex` ON `action`(`x`,`y`,`z`,`timestamp`,`actor`);\n",
+		// We run queries with the following filters:
+		// - `timestamp` >= ? AND `actor` = ?
+		// - `timestamp` >= ?
+		// - `timestamp` >= ? AND <range query on X, Y, Z>
+		"CREATE INDEX IF NOT EXISTS `actionIndex` ON `action`(`x`,`y`,`z`,`timestamp`,`actor`);\n"
+		"CREATE INDEX IF NOT EXISTS `actionTimestampActorIndex` ON `action`(`timestamp`,`actor`);\n",
 		NULL, NULL, NULL));
 	verbosestream << "SQL Rollback: SQLite3 database structure was created" << std::endl;
 
@@ -766,9 +771,7 @@ void RollbackManager::flush()
 {
 	sqlite3_exec(db, "BEGIN", NULL, NULL, NULL);
 
-	std::list<RollbackAction>::const_iterator iter;
-
-	for (iter  = action_todisk_buffer.begin();
+	for (auto iter = action_todisk_buffer.begin();
 			iter != action_todisk_buffer.end();
 			++iter) {
 		if (iter->actor.empty()) {
